{% extends 'lara_django_base/lara_base.html' %}
{% block toolbar %}
    <div class="d-flex justify-content-between flex-wrap flex-md-nowrap align-items-center pt-3 pb-2 mb-3 border-bottom">
        <h1 class="h2">{{ section_title }}</h1>
    </div>
{% endblock %}
{% load crispy_forms_tags %}
{% block content %}
    {% crispy form
    %}
{% endblock %}
